﻿using Microsoft.CSharp;
using System;
using System.CodeDom.Compiler;
using System.Reflection;
using System.Text;

namespace TestProject
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            Console.Title = "CSharpCodeProvider 클래스 : 동적 컴파일 사용하기";

            string code = @"
using System;
using System.Drawing;

namespace TestProject
{
    public class Program
    {
        public static void Main()
        {
            Point point = new Point(300, 200);

            Console.WriteLine(point.ToString());
        }
    }
}
";

            CSharpCodeProvider cSharpCodeProvider = new CSharpCodeProvider();
            CompilerParameters compilerParameters = new CompilerParameters();

            compilerParameters.ReferencedAssemblies.Add("System.Drawing.dll");

            compilerParameters.GenerateInMemory   = true;
            compilerParameters.GenerateExecutable = true;

            CompilerResults compilerResults = cSharpCodeProvider.CompileAssemblyFromSource
            (
                compilerParameters,
                code
            );

            if(compilerResults.Errors.HasErrors)
            {
                StringBuilder stringBuilder = new StringBuilder();

                foreach(CompilerError compilerError in compilerResults.Errors)
                {
                    stringBuilder.AppendLine
                    (
                        string.Format
                        (
                            "Error ({0}) : {1}",
                            compilerError.ErrorNumber,
                            compilerError.ErrorText
                        )
                    );
                }

                throw new InvalidOperationException(stringBuilder.ToString());
            }

            Assembly assembly = compilerResults.CompiledAssembly;

            Type programType = assembly.GetType("TestProject.Program");

            MethodInfo mainMethodInfo = programType.GetMethod("Main");

            mainMethodInfo.Invoke(null, null);
        }

        #endregion
    }
}